/* Copyright 2013 Perttu Luukko

 * This file is part of libeemd.

 * libeemd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * libeemd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with libeemd.  If not, see <http://www.gnu.org/licenses/>.
 */


#include <math.h>
#include <iostream>
#include <fstream>
#include <gsl/gsl_math.h>
const double pi = M_PI;

#include "../lib/eemd.h"

const size_t ensemble_size = 250;
const unsigned int S_number = 4;
const unsigned int num_siftings = 50;
const double noise_strength = 0.2;
const unsigned long int rng_seed = 0;
const char outfile[] = "eemd_example2.out";

// An example signal to decompose
const size_t N = 1024;
static inline double input_signal(double x) {
	const double omega = x/(N-1);
	return 2*sin(30*pi*omega) + 4.0*sin(20*pi*omega)*sin(0.2*pi*omega) + sin(10*pi*omega);
}

int main(void) {
	// Define input data
	double* inp = new double [N];
	for (size_t i=0; i<N; i++) {
		inp[i] = input_signal((double)i);
	}
	// Allocate memory for output data
	size_t M = emd_num_imfs(N);
	double* outp = new double [M*N];
	// Run eemd
	libeemd_error_code err = eemd(inp, N, outp, M, ensemble_size, noise_strength, S_number, num_siftings, rng_seed);
	if (err != EMD_SUCCESS) {
		emd_report_if_error(err);
		exit(1);
	}
	// Write output to file
    std::ofstream fp;
    fp.open(outfile);
	// Output in columns
	for (size_t j=0; j<N; j++) {
        fp << inp[j] << " ";
		for (size_t i=0; i<M; i++) {
            fp << outp[i*N+j] << " ";
		}
        fp << "\n";
	}
    std::clog << "Done!\n";
	// Cleanup
	fp.close();
	delete[] inp;
    delete[] outp;
}
